#  ------------------------------------------------------------------------#
# functions to model water-use data and population change, and fit and validate models----
#  ------------------------------------------------------------------------#


# load packages
invisible(sapply(c("magrittr", "quantreg", "stringr", "purrr", "broom", "tibble", "dplyr"),
                 require, character.only=T))

# set random seed
.seed = 42



#  ------------------------------------------------------------------------#
# fit models of water use using linear or quantile regression----
#  ------------------------------------------------------------------------#


# make function with following arguments from ".model_*" functions:
#   data = water-use data as data frame;
#   parm = parameters to model as character vector;
#   conlev = level for confidence interval as numeric vector
.fit_model = function(data=data,
                      parm=parm,
                      conlev=conlev) {

  if (parm == "Mean") {

    # fit linear regression of volume against number of wells
    mod = lm(Volume ~ N, data=data)

    # get parameter estimates and confidence intervals
    modparm = tidy(mod, conf.int=T, conf.level=conlev)

    # get model summary
    modsumm = glance(mod)

    # get predicted values, residuals, and confidence intervals
    modpred = augment(mod) %>%
      mutate(.conf.low = predict(mod, interval="confidence", level=conlev)[, "lwr"],
             .conf.high = predict(mod, interval="confidence", level=conlev)[, "upr"])
  } else {
    qile = str_extract(parm, "[[:digit:]]+") %>%
      as.numeric() %>%
      divide_by(100)

    # fit quantile regression of volume against number of wells
    mod = rq(Volume ~ N, data=data, tau=qile)

    # get parameter estimates and confidence intervals
    set.seed(.seed)
    modparm = tidy(mod, se.type="boot", conf.level=conlev, R=100)

    # get model summary
    modsumm = glance(mod)

    # get predicted values, residuals, and confidence intervals
    set.seed(.seed)
    modpred = augment(mod, type="percentile", interval="confidence", level=conlev, se="boot", R=100)
  }

  # combine predicted values with year variable from original data
  modpred = data %>%
    select(Year) %>%
    bind_cols(modpred)

  # make fitted model, predicted values, model summary, and parameter estimates
  dat = list()
  dat$Parameters = modparm
  dat$Summary = modsumm
  dat$Predictions = modpred
  dat$Model = mod
  return(dat)
}


#  ------------------------------------------------------------------------#
# validate models for predictions of water use with all possible combinations of observations using leave-one-out cross-validation----
#  ------------------------------------------------------------------------#


# make function with following arguments from ".model_*" functions:
#   data = water-use data as data frame;
#   parm = parameters to model as character vector;
#   conlev = level for confidence interval as numeric vector
.validate_model = function(data=data,
                           parm=parm,
                           conlev=conlev) {

  # for each cross-validated value
  datcros = data.frame("Predicted" = NA, "Observed" = NA)
  for (i in seq_len(nrow(data))) {

    # train model by fitting all but one left-out observation
    traindat = data[-i, ]
    if (parm == "Mean") {

      # fit linear regression of volume against number of wells
      trainmod = lm(Volume ~ N, data=traindat)
    } else {
      qile = str_extract(parm, "[[:digit:]]+") %>%
        as.numeric() %>%
        divide_by(100)

      # fit quantile regression of volume against number of wells
      trainmod = rq(Volume ~ N, data=data, tau=qile)
    }

    # test model by predicting one left-out observation
    testdat = data[i, ]
    testmod = predict(trainmod, newdata=testdat, level=conlev)
    datcros[i, "Predicted"] = testmod
    datcros[i, "Observed"] = testdat$Volume
  }

  # get goodness-of-fit metrics for predicted values against observed values
  rsq = cor(datcros$Observed, datcros$Predicted) ^ 2  # r-squared
  mae = mean(abs(datcros$Predicted - datcros$Observed), na.rm=T)  # mean absolute error
  rmse = sqrt(mean((datcros$Predicted - datcros$Observed) ^ 2, na.rm=T))  # root mean square error

  # make cross-validated values and goodness-of-fit metrics
  dat = list()
  dat$GoodnessofFit = data.frame(Rsq = rsq, MAE = mae, RMSE = rmse)
  dat$CrossValidation = as_tibble(datcros)
  return(dat)
}



#  ------------------------------------------------------------------------#
# B.1. model direct water-use data----
#  ------------------------------------------------------------------------#


# make function with following arguments:
#   data = direct water-use data as data frame;
#   parm = parameters ("Mean", "P10", "P50", "P90", etc.) to model as character vector;
#   use = uses of data ("Fracking", "Cementing", "Drilling", "Total_Direct", etc.) as character vector in same order as "data" argument;
#   conlev = level for confidence interval as numeric vector
.model_direct = function(data=datdir,
                         parm=c("Mean", "P10", "P50", "P90"),
                         use=c("Fracking", "Cementing", "Drilling", "Total_Direct"),
                         conlev=.conlev) {

  if (any(sapply(list(data, parm, use, conlev), is.null))) stop("one or more necessary arguments missing")

  # for each use
  datuse = list()
  for (iuse in use) {

    # skip missing uses and values
    if (!iuse %in% unique(data$Use)) next

    # get data for use
    dat = data %>%
      filter(Use == iuse)

    # for each parameter
    datparm = list()
    for (iparm in parm) {

      # fit model using linear or quantile regression, and validate model using leave-one-out cross-validation
      modfit = .fit_model(data=dat, parm=iparm, conlev=conlev)
      modval = .validate_model(data=dat, parm=iparm, conlev=conlev)
      datparm[[iparm]] = append(modfit, modval)
    }
    datuse[[iuse]] = datparm
  }

  # make direct water use model by use and parameter
  moddir = datuse
  write_rds(moddir, "Analyses/moddir.rds")
  return(moddir)
}


#  ------------------------------------------------------------------------#
# B.2. model indirect water-use data----
#  ------------------------------------------------------------------------#


# make function with following arguments:
#   data = indirect water-use data as data frame;
#   parm = parameters ("Mean", "P10", "P50", "P90", etc.) to model as character vector;
#   use = uses of data ("Total_Indirect", etc.) as character vector in same order as "data" argument;
#   type = types of data ("Fitted", "Lower_Limit", "Upper_Limit") as character vector;
#   conlev = level for confidence interval as numeric vector
.model_indirect = function(data=datind,
                           parm=c("Mean", "P10", "P50", "P90"),
                           use=c("Total_Indirect"),
                           type=c("Fitted", "Lower_Limit", "Upper_Limit"),
                           conlev=.conlev) {

  if (any(sapply(list(data, parm, use, type, conlev), is.null))) stop("one or more necessary arguments missing")

  # for each use
  datuse = list()
  for (iuse in use) {

    # for each type
    dattype = list()
    for (itype in type) {

      # skip missing uses and types
      if (!iuse %in% unique(data$Use)) next
      if (!itype %in% unique(data$Type)) next

      # get data for use and type
      dat = data %>%
        filter(Use == iuse,
               Type == itype)

      # for each parameter
      datparm = list()
      for (iparm in parm) {

        # fit model using linear or quantile regression, and validate model using leave-one-out cross-validation
        modfit = .fit_model(data=dat, parm=iparm, conlev=conlev)
        modval = .validate_model(data=dat, parm=iparm, conlev=conlev)
        datparm[[iparm]] = append(modfit, modval)
      }
      dattype[[itype]] = datparm
    }
    datuse[[iuse]] = dattype
  }

  # make indirect water use model by use, type and parameter
  modind = datuse
  write_rds(modind, "Analyses/modind.rds")
  return(modind)
}


#  ------------------------------------------------------------------------#
# B.3. model ancillary water-use data----
#  ------------------------------------------------------------------------#


# make function with following arguments:
#   data = ancillary water-use data as data frame;
#   parm = parameters ("Mean", "P10", "P50", "P90", etc.) to model as character vector;
#   use = uses of data ("Aquaculture", "Domestic", "Industrial", "Irrigation", "Livestock", "Mining", "Public Supply", "Thermoelectric Power") as character vector;
#   type = types of data ("Fitted", "Lower_Limit", "Upper_Limit") as character vector;
#   conlev = level for confidence interval as numeric vector
.model_ancillary = function(data=datanc,
                            parm=c("Mean", "P10", "P50", "P90"),
                            use=c("Aquaculture", "Domestic", "Industrial", "Irrigation", "Livestock", "Mining", "Public Supply", "Thermoelectric Power"),
                            type=c("Fitted", "Lower_Limit", "Upper_Limit"),
                            conlev=.conlev) {

  if (any(sapply(list(data, parm, use, type, conlev), is.null))) stop("one or more necessary arguments missing")

  # for each use
  datuse = list()
  for (iuse in use) {

    # for each type
    dattype = list()
    for (itype in type) {

      # skip missing uses and types
      if (!iuse %in% unique(data$Use)) next
      if (!itype %in% unique(data$Type)) next

      # get data for use and type
      dat = data %>%
        filter(Use == iuse,
               Type == itype)

      # for each parameter
      datparm = list()
      for (iparm in parm) {

        # fit model using linear or quantile regression, and validate model using leave-one-out cross-validation
        modfit = .fit_model(data=dat, parm=iparm, conlev=conlev)
        modval = .validate_model(data=dat, parm=iparm, conlev=conlev)

        datparm[[iparm]] = append(modfit, modval)
      }
      dattype[[itype]] = datparm
    }
    datuse[[iuse]] = dattype
  }

  # make ancillary water use model by use, type and parameter
  modanc = datuse
  write_rds(modanc, "Analyses/modanc.rds")
  return(modanc)
}



#  ------------------------------------------------------------------------#
# B. model population change data----
#  ------------------------------------------------------------------------#


# make function with following arguments:
#   data = population change data as data frame;
#   conlev = level for confidence interval as numeric vector
.model_change = function(data=datchng,
                         conlev=.conlev) {

  if (any(sapply(list(data, conlev), is.null))) stop("one or more necessary arguments missing")

  # fit linear regression of population against lagged number of wells
  form = lapply(c("N", "N1", "N2", "N3"), function(x) paste("Persons ~", x))
  mod = lapply(form, function(x) lm(as.formula(x), data=data))
  idx = lapply(mod, function(x) summary(x)$r.squared) %>%
    which.max()
  mod %<>% .[[idx]]

  # get parameter estimates and confidence intervals
  modparm = tidy(mod, conf.int=T, conf.level=conlev)

  # get model summary
  modsumm = glance(mod)

  # get predicted values, residuals, and confidence intervals
  modpred = augment(mod) %>%
    mutate(.conf.low = predict(mod, interval="confidence", level=conlev)[, "lwr"],
           .conf.high = predict(mod, interval="confidence", level=conlev)[, "upr"])

  # combine fitted data with year variable from original data
  modpred = data %>%
    select(Year, Persons) %>%
    full_join(modpred) %>%
    select(-.rownames)

  # make fitted model, predicted values, model summary, and parameter estimates
  dat = list()
  dat$Parameters = modparm
  dat$Summary = modsumm
  dat$Predictions = modpred
  dat$Model = mod

  # make population change model
  modchng = dat
  write_rds(modchng, "Analyses/modchng.rds")
  return(modchng)
}
