#  ------------------------------------------------------------------------#
# functions to visualize water-use data and population change----
#  ------------------------------------------------------------------------#


# load packages
invisible(sapply(c("magrittr", "plyr", "ggplot2", "stringr", "readr", "purrr", "tidyr", "tibble", "dplyr"),
                 require, character.only=T))



#  ------------------------------------------------------------------------#
# make table of water use from linear and quantile regression models----
#  ------------------------------------------------------------------------#


# make function with following arguments from ".visualize_*" functions:
#   data = water-use model as data frame;
#   comp = component of model as character vector
.table_model = function(data=data,
                        comp=comp) {

  # get parameter estimates and confidence intervals
  dat = list()
  for (iuse in unique(names(data))) {
    dat[[iuse]] = modify_depth(data[[iuse]], .depth=ifelse(comp == "direct", 1, 2), .f="Parameters") %>%
      as.data.frame() %>%
      select(-matches(paste(c("error", "statistic", "value", "tau"), collapse="|")),
             -matches("[[:digit:]]+.term"))
  }
  if (comp == "direct") {
    dat %<>%
      ldply(data.frame, .id="Use") %>%
      filter(str_detect(Mean.term, "N")) %>%
      as_tibble()
  } else {
    dat %<>%
      ldply(data.frame, .id="Use") %>%
      filter(str_detect(Fitted.Mean.term, "N")) %>%
      t() %>%
      as.data.frame()
    dat %<>% rownames_to_column("ParType")
    names(dat) = c(names(dat)[1], unlist(dat[1, -1]))
    dat %<>%
      slice(-1) %>%
      gather(key=Use, value=Num, -ParType)
    dat %<>%
      mutate(Type = str_split_fixed(dat$ParType, "\\.", n=2)[, 1],
             Parameter = str_split_fixed(dat$ParType, "\\.", n=2)[, 2]) %>%
      select(-ParType) %>%
      spread(Parameter, Num) %>%
      arrange(Type)
    dat %<>% .[, c(1, 2, 6:3, 9:7, 12:10, 15:13)]
  }

  # make table of parameter estimates and confidence intervals
  dat[1, "Comments"] = "units in million gallons water use per well"
  write_excel_csv(dat, paste0("Analyses/", comp, "_water_use.csv"))
}


#  ------------------------------------------------------------------------#
# make figure of water use from linear and quantile regression models----
#  ------------------------------------------------------------------------#


# make function with following arguments from ".visualize_*" functions:
#   data = water-use model as data frame;
#   comp = component of model as character vector;
#   use = uses of data as character vector;
#   type = types of data as character vector;
#   conlev = level for confidence interval as numeric vector
.figure_model = function(data=data,
                         comp=comp,
                         use=use,
                         type=type,
                         conlev=conlev) {

  # for each parameter
  dat = temp = list()
  iparm0 = NULL
  for (iparm in unique(names(data))) {

    # get predicted values and parameter estimates from model
    dat[[iparm]] = data[[iparm]]$Predictions
    temp = data[[iparm]]$Parameters

    # drop unnecessary variables
    dat[[iparm]] %<>% select(Year, Volume, N, .fitted, .conf.low, .conf.high)

    # get parameter, use, and confidence level for labeling
    if (str_detect(iparm, "[[:digit:]]")) {
      iparm %<>%
        str_extract("[[:digit:]]+") %>%
        paste0("th percentile")
    }
    iparm0 %<>% c(iparm)
  }
  iparm0 %<>% str_to_lower()
  use0 = use %>%
    str_replace("_", " ") %>%
    str_to_sentence()
  conlev0 = paste0(conlev * 100, "% confidence intervals")

  # combine data frames
  names(dat) = paste("Fitted", iparm0)
  dat %<>% bind_rows(.id="Parameter")

  # order parameters for labeling
  brk = dat %>%
    group_by(Parameter) %>%
    summarize(.fitted = max(.fitted)) %>%
    arrange(desc(.fitted)) %>%
    pull(Parameter)

  # make figure of water use against wells
  ggp = ggplot(dat) +
    geom_ribbon(aes(N, ymin=.conf.low, ymax=.conf.high, group=Parameter, fill=conlev0), alpha=0.5) +
    geom_ribbon(aes(N, ymin=.conf.low, ymax=.conf.high, group=Parameter, fill=Parameter), alpha=0.5) +
    geom_line(aes(N, .fitted, color=Parameter)) +
    geom_point(aes(N, Volume, color="Observation"), size=0.5) +
    theme_gray(base_size=9) +
    scale_color_manual(name="EXPLANATION", values=c("#d73027", "#f46d43", "#fdae61", "#4575b4", "#313695"), breaks=c("Observation", brk)) +
    scale_fill_manual(name="", values=c("gray50", "#d73027", "#f46d43", "#fdae61", "#4575b4"), breaks=conlev0) +
    labs(title=paste(use0, "water use"), x="Wells", y="Volume (Mgal)", caption="[Mgal, million gallons]") +
    theme(legend.margin=margin(0, 0, 0, 0, "inch"), legend.spacing=unit(0.1, "inch")) +
    guides(color=guide_legend(override.aes=list(linetype=c(0, rep(1, times=length(brk))), shape=c(16, rep(NA, times=length(brk))), size=c(1.5, rep(0.5, times=length(brk)))), order=1))
  if (comp == "direct") {
    ggsave(paste0("Plots/", comp, "_water_use_", use, ".pdf"), width=6.5, height=3.5)
  } else {
    ggsave(paste0("Plots/", comp, "_water_use_", use, "_", type, ".pdf"), width=6.5, height=3.5)
  }
  return(ggp)
}



#  ------------------------------------------------------------------------#
# C.1. visualize direct water-use data----
#  ------------------------------------------------------------------------#


# make function with following arguments:
#   data = direct water-use parameters from ".model_direct" function as list;
#   comp = water-use component ("direct") for modeling as character vector;
#   conlev = level for confidence interval as numeric vector
.visualize_direct = function(data=moddir,
                             comp="direct",
                             conlev=.conlev) {

  if (any(sapply(list(data, comp, conlev), is.null))) stop("one or more necessary arguments missing")

  # make table of parameter estimates and confidence intervals
  vis = .table_model(data=data,
                     comp=comp)
  write_rds(vis, paste0("Analyses/vistbldir.rds"))

  # for each use
  vis = list()
  for (iuse in unique(names(data))) {
    data2 = data[[iuse]]

    # make figures of water use against wells
    vis[[iuse]] = .figure_model(data=data2,
                                comp=comp,
                                use=iuse,
                                type=NULL,
                                conlev=conlev)
  }
  write_rds(vis, paste0("Analyses/visfigdir.rds"))
}


#  ------------------------------------------------------------------------#
# C.2. visualize indirect water-use data----
#  ------------------------------------------------------------------------#


# make function with following arguments:
#   data = indirect water-use parameters from ".model_indirect" function as list;
#   comp = water-use component ("indirect") for modeling as character vector;
#   conlev = level for confidence interval as numeric vector
.visualize_indirect = function(data=modind,
                               comp="indirect",
                               conlev=.conlev) {

  if (any(sapply(list(data, comp, conlev), is.null))) stop("one or more necessary arguments missing")

  # make table of model coefficients and confidence interval
  vis = .table_model(data=data,
                     comp=comp)
  write_rds(vis, paste0("Analyses/vistblind.rds"))

  # for each use and type
  vis = list()
  for (iuse in unique(names(data))) {
    for (itype in unique(names(data[[iuse]]))) {
      data2 = data[[iuse]][[itype]]

      # make figures of water use against wells
      vis[[iuse]][[itype]] = .figure_model(data=data2,
                                           comp=comp,
                                           use=iuse,
                                           type=itype,
                                           conlev=conlev)
    }
  }
  write_rds(vis, paste0("Analyses/visfigind.rds"))
}


#  ------------------------------------------------------------------------#
# C.3. visualize ancillary water-use data----
#  ------------------------------------------------------------------------#


# make function with following arguments:
#   data = ancillary water-use parameters  from ".model_indirect" function as list;
#   comp = water-use component ("ancillary") for modeling as character vector;
#   conlev = level for confidence interval as numeric vector
.visualize_ancillary = function(data=modanc,
                                comp="ancillary",
                                conlev=.conlev) {

  if (any(sapply(list(data, comp, conlev), is.null))) stop("one or more necessary arguments missing")

  # make table of model coefficients and confidence interval
  vis= .table_model(data=data,
                    comp=comp)
  write_rds(vis, paste0("Analyses/vistblanc.rds"))

  # for each use and type
  vis = list()
  for (iuse in unique(names(data))) {
    for (itype in unique(names(data[[iuse]]))) {
      data2 = data[[iuse]][[itype]]

      # make figures of water use against wells
      vis[[iuse]][[itype]] = .figure_model(data=data2,
                                           comp=comp,
                                           use=iuse,
                                           type=itype,
                                           conlev=conlev)
    }
  }
  write_rds(vis, paste0("Analyses/visfiganc.rds"))
}



#  ------------------------------------------------------------------------#
# C. visualize population change data----
#  ------------------------------------------------------------------------#


# make function with following arguments:
#   data = population change from ".model_population" function as list;
#   conlev = level for confidence interval as numeric vector
.visualize_change = function(data=modchng,
                             conlev=.conlev) {

  if (any(sapply(list(data, conlev), is.null))) stop("one or more necessary arguments missing")

  # make table of model coefficients and confidence intervals
  dat =  data$Parameters %>%
    filter(str_detect(term, "N")) %>%
    select(term, estimate, conf.low, conf.high)
  dat[1, "Comments"] = "units in persons per well (lagged by number of years shown by 'N')"
  write_excel_csv(dat, "Analyses/population_change.csv")

  # get confidence level for labeling
  conlev0 = paste0(conlev * 100, "% confidence interval")

  # get data from model
  dat = data$Data
  names(dat)[str_which(names(dat), "N")] = "N"

  # make figures of population change against wells and years
  ggp = ggplot(dat) +
    geom_ribbon(aes(N, ymin=.conf.low, ymax=.conf.high, fill=conlev0), alpha=0.5) +
    geom_line(aes(N, .fitted, color="Fitted model")) +
    geom_point(aes(N, Persons, color="Observation"), size=1) +
    theme_gray(base_size=9) +
    scale_color_manual(name="EXPLANATION", values=c("#1b7837", "#40004b"), breaks=c("Observation", "Fitted model")) +
    scale_fill_manual(name="", values="#1b7837") +
    labs(title="Population change", x="Wells", y="Persons") +
    theme(legend.margin=margin(0, 0, 0, 0, "inch"), legend.spacing=unit(0.1, "inch")) +
    guides(color=guide_legend(title.position="top", override.aes=list(linetype=c(0, 1), shape=c(16, NA), size=c(1.5, 0.5)), order=1))
  ggsave("Plots/population_change_wells.pdf", width=6.5, height=3.5)
  ggp = ggplot(dat) +
    geom_ribbon(aes(Year, ymin=.conf.low, ymax=.conf.high, fill=conlev0), alpha=0.5) +
    geom_line(aes(Year, .fitted, color="Fitted model")) +
    geom_point(aes(Year, Persons, color="Observation"), size=1) +
    theme_gray(base_size=9) +
    scale_x_continuous(breaks=function(x) seq(ceiling(x[1]), floor(x[2]), by=1)) +
    scale_color_manual(name="EXPLANATION", values=c("#1b7837", "#40004b"), breaks=c("Observation", "Fitted model")) +
    scale_fill_manual(name="", values="#1b7837") +
    labs(title="Population change", x="Year", y="Persons") +
    theme(legend.margin=margin(0, 0, 0, 0, "inch"), legend.spacing=unit(0.1, "inch")) +
    guides(color=guide_legend(title.position="top", override.aes=list(linetype=c(0, 1), shape=c(16, NA), size=c(1.5, 0.5)), order=1))
  ggsave("Plots/population_change_years.pdf", width=6.5, height=3.5)
}
