#  ------------------------------------------------------------------------#
# functions to wrangle water-use data, and model breakpoint----
#  ------------------------------------------------------------------------#


# load packages
invisible(sapply(c("magrittr", "segmented", "plyr", "stringr", "purrr", "tidyr", "tibble", "dplyr"),
                 require, character.only=T))



#  ------------------------------------------------------------------------#
# model year that COG development began by estimating breakpoint in volume of water used using segmented linear regression----
#  ------------------------------------------------------------------------#


# make function with following arguments:
#   data = water-use data as data frame
.model_breakpoint = function(data) {

  # summarize volume by year
  dat = data %>%
    group_by(Year) %>%
    summarize(Volume = sum(Volume))

  # fit linear regression of volume against year
  mod = lm(Volume ~ Year, data=dat)

  # estimate breakpoint by fitting segmented relationship to fitted linear model of volume against year
  brkpt = segmented(mod, seg.Z=~Year, psi=list(Year=NA), control=seg.control(stop.if.error=F, n.boot=0, it.max=100))
  if (is.null(brkpt$psi)) {
    stop("no breakpoint estimated; use judgment to choose appropriate year that COG development began")
  } else if (nrow(brkpt$psi) > 1) {
    print(round(brkpt$psi[,"Est."]))
    stop("multiple breakpoints estimated; use judgment to choose appropriate year that COG development began")
  } else {
    print(b)
    return(round(brkpt$psi[,"Est."]))
  }
}



#  ------------------------------------------------------------------------#
# A.1. wrangle direct water-use data----
#  ------------------------------------------------------------------------#


# make function with following arguments:
#   data = direct water-use data for fracking, cementing, drilling or other uses as data frames;
#   brkpt = estimated breakpoint from ".model_breakpoint" function as numeric vector;
#   use = uses of data ("Fracking", "Cementing", "Drilling", etc.) as character vector in same order as "data" argument;
#   type = types of data ("Observed", "Estimated", "Fitted", "Applied", etc.) as character vector in same order as "data" argument
.wrangle_direct = function(data=list(datfrac, datcemt, datdril),
                           brkpt=.brkpt,
                           use=c("Fracking", "Cementing", "Drilling"),
                           type=c("Observed", "Estimated", "Estimated")) {

  if (any(sapply(list(data, brkpt, use, type), is.null))) stop("one or more necessary arguments missing")

  # for each use
  datdir = list()
  for (i in seq_along(data)) {

    # summarize volume and number of wells by state, year and county for years following estimated breakpoint
    datdir[[use[i]]] = data[[i]] %>%
      filter(Year >= brkpt) %>%
      group_by(State, County, Year) %>%
      summarize(Volume = sum(Volume),
                N = n()) %>%
      mutate(Type = type[[i]]) %>%
      arrange(Year)
  }

  # summarize volume and number of wells by state, year and county for total direct uses
  datdir[["Total_Direct"]] = reduce(datdir, full_join) %>%
    group_by(State, County, Year) %>%
    summarize(Volume = sum(Volume),
              N = first(N)) %>%
    arrange(Year)

  # make direct water-use data by use
  datdir %<>%
    ldply(data.frame, .id="Use") %>%
    as_tibble()
  write_rds(datdir, "Analyses/datdir.rds")
  return(datdir)
}


#  ------------------------------------------------------------------------#
# A.2. wrangle indirect water-use data----
#  ------------------------------------------------------------------------#


# make function with following arguments:
#   data = permit water-use data and other data as data frames;
#   data2 = direct water-use data from ".wrangle_direct" function as data frame;
#   brkpt = estimated breakpoint from ".model_breakpoint" function as numeric vector;
#   cogrel = whether permits are related to COG ("Yes", "Maybe", etc.) as character vector;
#   wduse = whether permits have use of "Water Depot" (or NULL) as character vector;
#   conlev = level for confidence interval as numeric vector
.wrangle_indirect = function(data=list(datperm, dattemp),
                             data2=datdir,
                             brkpt=.brkpt,
                             cogrel=c("Yes", "Maybe"),
                             wduse="Water Depot",
                             conlev=.conlev) {

  if (any(sapply(list(data, data2, brkpt, cogrel, conlev), is.null))) stop("one or more necessary arguments missing")

  # get total water use (combination of direct and indirect water use) from water permits or other indirect water-use data
  dattot = reduce(data, full_join)

  # get total water use for permits related to COG or with use of "Water Depot"
  dattot %<>% filter(RelatedtoCOG %in% cogrel)
  if (!is.null(wduse)) {
    dattot %<>% filter(str_detect(Use, fixed(wduse, ignore_case=T)))
  }

  # summarize total volume by state and year for 15 years preceding (pre-COG) or years following (post-COG) estimated breakpoint
  dattotpre = dattotpost = dattot %>%
    group_by(State, Year) %>%
    summarize(Volume = sum(Volume))
  dattotpre %<>% filter(between(Year, brkpt - 15, brkpt - 1))
  dattotpost %<>% filter(Year >= brkpt)

  # fit linear regression of volume against year for pre-COG years
  mod = lm(Volume ~ Year, data=dattotpre)

  # get confidence interval around sample values for post-COG years
  pred = predict(mod, newdata=dattotpost, interval="confidence", level=conlev) %>%
    as.data.frame()

  # get total volume by year for post-COG years as residual value (difference between fitted and confidence limit values and observed values)
  dattotpost %<>%
    bind_cols(pred) %>%
    mutate(Fitted = Volume - fit,
           Lower_Limit = Volume - upr,
           Upper_Limit = Volume - lwr) %>%
    select(-c(fit, lwr, upr))

  # get total direct water use, and summarize direct volume by state and year
  datdir = data2 %>%
    filter(Use == "Total_Direct") %>%
    group_by(State, Year) %>%
    summarize(Volume = sum(Volume))

  # get indirect volume by year for post-COG years as residual value (difference between total volume and direct volume)
  datind = full_join(dattotpost, datdir, by=c("State", "Year")) %>%
    mutate(Fitted = Fitted - Volume.y,
           Lower_Limit = Lower_Limit - Volume.y,
           Lower_Limit = ifelse(Lower_Limit < 0, 0, Lower_Limit),  # lower confidence limit must be at least zero
           Upper_Limit = ifelse(Upper_Limit > Volume.x, Volume.x, Upper_Limit),  # upper confidence limit value must be at most observed value
           Upper_Limit = Upper_Limit - Volume.y) %>%
    select(-contains("Volume"))

  # get total direct water use, and summarize direct volume and number of wells by state, year and county
  datdir = data2 %>%
    filter(Use == "Total_Direct") %>%
    group_by(State, Year, County) %>%
    summarize(Volume = sum(Volume),
              N = first(N))

  # summarize number of wells by state and year
  datwel = datdir %>%
    group_by(State, Year) %>%
    summarize(N = sum(N)) %>%
    uncount(weights=table(match(datdir$Year, datind$Year)))

  # make indirect volume by state and year to indirect volume by state, year and county for post-COG years as proportional value (ratio of number of wells in any county to number of wells in all state)
  datind %<>%
    uncount(weights=table(match(datdir$Year, datind$Year))) %>%
    bind_cols(datdir, datwel) %>%
    select(State, County, Year, Fitted, Lower_Limit, Upper_Limit, N, N1) %>%
    gather(key=Type, value=Volume, -c(State, County, Year, N, N1)) %>%
    mutate(Volume = Volume * N / N1) %>%
    select(State, County, Year, Volume, N, Type)

  # make indirect water-use data for total indirect uses
  datind %<>%
    ungroup() %>%
    mutate(Use = "Total_Indirect") %>%
    select(Use, everything())
  write_rds(datind, "Analyses/datind.rds")
  return(datind)
}


#  ------------------------------------------------------------------------#
# A.3. wrangle ancillary water-use data----
#  ------------------------------------------------------------------------#


# make function with following arguments:
#   data = permit water-use data and other data as data frames;
#   data2 = direct water-use data from ".wrangle_direct" function as data frame;
#   data3 = modeled climate data as data frame;
#   brkpt = estimated breakpoint from ".model_breakpoint" function as numeric vector;
#   cogrel = whether permits are related to COG ("Yes", "Maybe", etc.) as character vector;
#   wduse = whether permits have use of "Water Depot" (or NULL) as character vector;
#   conlev = level for confidence interval as numeric vector
.wrangle_ancillary = function(data=list(datperm, dattemp),
                              data2=datdir,
                              data3=datclim,
                              brkpt=.brkpt,
                              cogrel=c("Yes", "Maybe"),
                              wduse="Water Depot",
                              conlev=.conlev) {

  if (any(sapply(list(data, data2, data3, brkpt, cogrel, conlev), is.null))) stop("one or more necessary arguments missing")

  # get total water use (combination of direct, indirect and ancillary water use) from water permits or other ancillary water-use data
  dattot = reduce(data, full_join)

  # get county/state combinations with at least 5 water permits related to COG for years following estimated breakpoint
  stcnty = dattot %>%
    filter(RelatedtoCOG %in% cogrel,
           Year >= brkpt) %>%
    group_by(State, County) %>%
    summarize(N = n()) %>%
    filter(N >= 5) %>%
    select(State, County)

  # get precipitation and temperature data for state/county combinations associated with COG
  datclim = data3 %>%
    filter(State %in% stcnty$State & County %in% stcnty$County) %>%
    group_by(State, Year) %>%
    summarize(Precip = mean(Precip),
              MeanTemp = mean(MeanTemp))

  # get ancillary water use for state/county combinations associated with COG
  datanc = dattot %>%
    filter(State %in% stcnty$State,
           County %in% stcnty$County)
  if (!is.null(wduse)) {
    datanc %<>% filter(!str_detect(Use, fixed(wduse, ignore_case=T)))
  } else {
    datanc %<>% filter(!RelatedtoCOG %in% cogrel)
  }

  # combine ancillary water use and climate
  datanc %<>% left_join(datclim)

  # for each use
  dat = list()
  for (iuse in unique(datanc$Use)) {

    # skip uses without USGS category
    if (!iuse %in% c("Aquaculture", "Domestic", "Industrial", "Irrigation", "Livestock", "Mining", "Public Supply", "Thermoelectric Power")) next

    # summarize ancillary volume by state and year for 15 years preceding (pre-COG) or years following (post-COG) estimated breakpoint
    datancpre = datancpost = datanc %>%
      filter(Use == iuse) %>%
      group_by(State, Year) %>%
      summarize(Volume = sum(Volume),
                Precip = mean(Precip),
                MeanTemp = mean(MeanTemp))
    datancpre %<>% filter(between(Year, brkpt - 15, brkpt - 1))
    datancpost %<>% filter(Year >= brkpt)

    # skip uses without enough data
    if (nrow(datancpre) < 5 | nrow(datancpost) < 5) next

    # fit linear regressions of response against all possible combinations of predictors for pre-COG years
    vrbs = c("Year", "Precip", "MeanTemp")
    idx = unlist(lapply(1:length(vrbs), function(x) combn(1:length(vrbs), x, simplify=F)), recursive=F)
    form = lapply(idx, function(x) paste("Volume ~", paste(vrbs[x], collapse=" + ")))
    mod = lapply(form, function(x) lm(as.formula(x), data=datancpre))

    # get best model for pre-COG years
    idx = which.max(lapply(mod, function(x) summary(x)$adj.r.squared))
    mod %<>% .[[idx]]

    # get confidence interval around sample values for post-COG years
    pred = predict(mod, newdata=datancpost, interval="confidence", level=conlev) %>%
      as.data.frame()

    # get ancillary volume by year for post-COG years as residual value (difference between fitted and confidence limit values and observed values)
    datancpost %<>%
      bind_cols(pred) %>%
      mutate(Fitted = Volume - fit,
             Lower_Limit = Volume - upr,
             Upper_Limit = Volume - lwr) %>%
      select(-c(fit, lwr, upr))

    # get total direct water use, and summarize number of wells by year and county
    datdir = data2 %>%
      filter(Use == "Total_Direct") %>%
      group_by(Year, County) %>%
      summarize(N = first(N))

    # summarize number of wells by state and year
    datwel = datdir %>%
      group_by(Year) %>%
      summarize(N = sum(N)) %>%
      uncount(weights=table(match(datdir$Year, datancpost$Year)))

    # make ancillary volume by state and year to ancillary volume by state, year and county for post-COG years as proportional value (ratio of number of wells in any county to number of wells in all state)
    dat[[iuse]] = datancpost %>%
      uncount(weights=table(match(datdir$Year, datancpost$Year))) %>%
      bind_cols(datdir, datwel) %>%
      select(State, County, Year, Fitted, Lower_Limit, Upper_Limit, N, N1) %>%
      gather(key=Type, value=Volume, -c(State, County, Year, N, N1)) %>%
      mutate(Volume = Volume * N / N1) %>%
      select(State, County, Year, Volume, N, Type)
  }

  # make ancillary water-use data by use
  datanc = dat %>% ldply(data.frame, .id="Use") %>%
    as_tibble()
  write_rds(datanc, "Analyses/datanc.rds")
  return(datanc)
}



#  ------------------------------------------------------------------------#
# A. wrangle population change data----
#  ------------------------------------------------------------------------#


# make function with following arguments:
#   data = permit water-use data and other data as data frames;
#   data2 = direct water-use data from ".wrangle_direct" function as data frame;
#   data3 = estimated population data as data frame;
#   brkpt = estimated breakpoint from ".model_breakpoint" function as numeric vector;
#   cogrel = whether permits are related to COG ("Yes", "Maybe", etc.) as character vector;
#   conlev = level for confidence interval as numeric vector
.wrangle_change = function(data=list(datperm, dattemp),
                           data2=datdir,
                           data3=datpopn,
                           brkpt=.brkpt,
                           cogrel=c("Yes", "Maybe"),
                           conlev=.conlev) {

  if (any(sapply(list(data, data2, data3, brkpt, cogrel, conlev), is.null))) stop("one or more necessary arguments missing")

  # get total water use (combination of direct, indirect and ancillary water use) from water permits or other ancillary water-use data
  dattot = reduce(data, full_join)

  # get county/state combinations with at least 5 water permits related to COG for years following estimated breakpoint
  stcnty = dattot %>%
    filter(RelatedtoCOG %in% cogrel,
           Year >= brkpt) %>%
    group_by(State, County) %>%
    summarize(N = n()) %>%
    filter(N >= 5) %>%
    select(State, County)

  # get population estimates for state/county combinations associated with COG
  datchng = data3 %>%
    filter(State %in% stcnty$State & County %in% stcnty$County) %>%
    group_by(State, Year) %>%
    summarize(Persons = sum(Persons))

  # make population estimates by state and year for post-COG years with number of wells lagged by zero through three years
  datchng %<>%
    right_join(data2 %>%
                 filter(Use == "Total_Direct",
                        Year >= .brkpt) %>%
                 group_by(State, Year) %>%
                 summarize(N = sum(N))) %>%
    mutate(N1 = lag(N, 1),
           N2 = lag(N, 2),
           N3 = lag(N, 3))

  # make population change data
  datchng %<>% ungroup()
  write_rds(datchng, "Analyses/datchng.rds")
  return(datchng)
}
