#  ------------------------------------------------------------------------#
# munge raw data release files for running water use analysis----
#  ------------------------------------------------------------------------#


rm(list=ls())

# set global options
options(stringsAsFactors=F)

# load packages (install if necessary)
invisible(sapply(c("magrittr", "raster", "sf", "stringr", "readr", "purrr", "tidyr", "tibble", "dplyr"),
                 function(x) {
                   if (!require(x, character.only=T)) {
                     install.packages(x)
                     library(x, character.only=T)
                   }
                 }))



#  ------------------------------------------------------------------------#
# make working data files with appropriate structure----
#  ------------------------------------------------------------------------#


# change variable names, add missing variable for state and type, and change use to USGS categories for permit water use data from ND State Water Commission
dat = read_csv("Raw/NDSWCreported.csv") %>%
  rename(Source = SourceType,
         Use = UseType,
         Volume = Reported) %>%
  mutate(State = "North Dakota",
         Type = "Unknown") %>%
  select(State, County, Source, Type, Use, RelatedtoCOG, Year, Volume)
dat$Source %<>% recode("Surface Water" = "Surface",
                       "Ground Water" = "Ground")
dat$Use %<>% recode("Fish and Wildlife" = "Aquaculture",
                    "Flood Control" = "Industrial",
                    "Industrial - Water Depot" = "Mining Water Depot",
                    "Multiple Use" = "Unspecified",
                    "Municipal" = "Public Supply",
                    "Power Generation" = "Thermoelectric Power",
                    "Recreation" = "Public Supply",
                    "Repurposed - Water Depot" = "Mining Water Depot",
                    "Rural Water" = "Domestic",
                    "Stock" = "Livestock")
dat %<>% mutate(Use = case_when(Use == "Industrial" & RelatedtoCOG == "Yes" ~ "Mining",
                                TRUE ~ Use))
write_csv(dat, "Data/permit_water_used.csv")

# change variable names, add missing variable for state and type, and change use to USGS categories for temporary permit water use data from ND State Water Commission
dat = read_csv("Raw/NDSWCtempreported.csv") %>%
  rename(Source = SourceType,
         Use = UseType,
         Volume = Reported) %>%
  mutate(State = "North Dakota",
         Type = "Unknown") %>%
  select(State, County, Source, Type, Use, RelatedtoCOG, Year, Volume)
dat$Source %<>% recode("Surface Water" = "Surface",
                       "Ground Water" = "Ground")
dat$Use %<>% recode("Construction" = "Industrial",
                    "Industrial - Water Depot" = "Mining Water Depot",
                    "Municipal" = "Public Supply",
                    "Resource Planning" = "Unspecified",
                    "Rural Water" = "Domestic")
dat %<>% mutate(Use = case_when(Use == "Industrial" & RelatedtoCOG == "Yes" ~ "Mining",
                                TRUE ~ Use))
write_csv(dat, "Data/temporary_permit_water_used.csv")

# add missing variable for state for well fracking water use data from ND Industrial Commission
dat = read_csv("Raw/NDICstimulation.csv") %>%
  mutate(State = "North Dakota") %>%
  select(State, County, Year, Volume)
write_csv(dat, "Data/well_fracking_water_used.csv")

# add missing variables for state and volume for well cementing water use data from ND Industrial Commission (estimated as per personal communication)
dat = read_csv("Raw/NDICcementing.csv") %>%
  mutate(State = "North Dakota",
         Volume = Sacks * 7 / 42) %>%  # 7 gallons per sack
  select(State, County, Year, Volume)
write_csv(dat, "Data/well_cementing_water_used.csv")

# add missing variables for state and volume for well drilling water use data from ND Industrial Commission (estimated as per personal communication)
dat = read_csv("Data/well_cementing_water_used.csv") %>%
  mutate(Volume = Volume * 0.5)
write_csv(dat, "Data/well_drilling_water_used.csv")  # drilling at 50% of volume for cementing water used

# change variable name for estimated population data from U.S. Census Bureau
dat = read_csv("Raw/USCBpopulation.csv") %>%
  rename(Persons = Population) %>%
  filter(State == "North Dakota") %>%
  select(State, County, Year, Persons)
write_csv(dat, "Data/population_estimated.csv")

# read cartographic boundary shapefile from U.S. Census Bureau
fil = "cb_2017_us_county_20m"
download.file(paste0("https://www2.census.gov/geo/tiger/GENZ2017/shp/", fil, ".zip"), destfile=paste0("Raw/", fil, ".zip"), method="curl")
unzip(paste0("Raw/", fil, ".zip"), exdir=paste0("Raw/", fil))
cb = st_read(Sys.glob("Raw/cb*/*.shp"))

# get spatial features for counties in states
fips = data.frame(name = c("North Dakota"),
                  code = c(38))
cb %<>% filter(STATEFP %in% fips$code)
cb %<>% mutate(name = fips$name[match(cb$STATEFP, fips$code)])

# get modeled climate data for precipitation and temperature variables from PRISM Climate Group
vrb = c("ppt", "tmin", "tmax", "tmean")
cl = list()
for (ivrb in vrb) {

  # read climate variable rasters
  unzip(paste0("Raw/", ivrb, ".zip"), exdir=paste0("Raw/", ivrb))
  fil = Sys.glob(paste0("Raw/", ivrb, "/*.zip")) %>%
    str_split("\\/", simplify=T) %>%
    .[, 3] %>%
    str_split(fixed("."), simplify=T) %>%
    .[, 1]
  sapply(fil, function(x) unzip(paste0("Raw/", ivrb, "/", x, ".zip"), exdir=paste0("Raw/", x)))

  # make raster stack of years for climate variable
  fil = Sys.glob(paste0("Raw/PRISM_", ivrb, "*/*bil.bil")) %>%
    .[-str_which(., "[[:digit:]]{6}")] %>%
    stack()

  # extract mean value of variable by counties of states
  cl[[ivrb]] = raster::extract(fil, cb, fun=mean, na.rm=T) %>%
    t() %>%
    as.data.frame() %>%
    set_names(paste(cb$name, cb$NAME, sep="_")) %>%
    rownames_to_column(var="Year") %>%
    mutate(Year = str_split(.$Year, "_", simplify=T)[, 5]) %>%
    gather(key = "County", value = !!ivrb, -Year) %>%
    mutate(State = str_split(.$County, "_", simplify=T)[, 1]) %>%
    mutate(County = str_split(.$County, "_", simplify=T)[, 2]) %>%
    select(State, County, everything())
}

# combine precipitation and temperature variables
cl %<>%
  reduce(full_join) %>%
  rename(Precip = ppt,
         MinTemp = tmin,
         MaxTemp = tmax,
         MeanTemp = tmean) %>%
  arrange(State, County)

# write data for analysis
write_csv(cl, "Raw/PRISMclimate.csv")

# add variable for state for modeled climate data from PRISM Climate Group
dat = read_csv("Raw/PRISMclimate.csv") %>%
  filter(State == "North Dakota") %>%
  select(State, County, Year, Precip, MeanTemp, MinTemp, MaxTemp)
write_csv(dat, "Data/climate_modeled.csv")
