#  ------------------------------------------------------------------------#
# analyze water use associated with continuous oil and gas (COG) development----
#  ------------------------------------------------------------------------#


## NOTE:
#   1) run the following code chunk once at beginning of water-use analysis
#   2) see the following "NOTE"s (sections B.1, B.2, B.3, C.1, C.2, and C.3) for how the "wrangle.R", "model.R", and "visualize.R" scripts may be rewritten as needed

rm(list=ls())

# set global options
options(stringsAsFactors=F)

# load packages (install if necessary)
invisible(sapply(c("magrittr", "raster", "sf", "segmented", "quantreg", "plyr", "ggplot2", "stringr", "readr", "purrr", "broom", "tidyr", "tibble", "dplyr"),
                 function(x) {
                   if (!require(x, character.only=T)) {
                     install.packages(x)
                     library(x, character.only=T)
                   }
                 }))

# load functions
invisible(sapply(paste0("Functions/", c("wrangle", "model", "visualize"), ".R"),
                 source))



#  ------------------------------------------------------------------------#
# A.0. initialize input to water-use analysis----
#  ------------------------------------------------------------------------#


## NOTE:
#   1) run the following code line once at beginning of water-use analysis
#   2) once run, comment the code line to not run again
#   3) see the following "NOTE"s (sections A.1., A.2., and A.3.) for how the "munge_data_release.R" script is structuring the data

# munge raw data release files in "Raw" folder into working data files in "Data" folder
source("munge_data_release.R")


#  ------------------------------------------------------------------------#
# A.1. input to direct water-use analysis----
#  ------------------------------------------------------------------------#


## NOTE:
#   1) to run the following code chunks (including sections B.1. and C.1.) as is, the data must be structured as follows:
#     a) rows of observations for individual oil and gas wells;
#     b) columns of variables for state, county, year and volume
#     c) see "well_fracking_water_used.csv" for the appropriate data structure
#   2) the script may throw an error if data have the following limitations:
#     a) missing one or more of the listed variables;
#     b) lacking observations by individual well (for example, observations may be aggregated by county or by unit area)
#   3) the script can be modified as appropriate to run in the following conditions:
#     a) a COG play may have data available at a finer temporal resolution, such as by month;
#     b) a COG play likely will not have data available on water used in cementing or drilling of oil and gas wells

# read data for water used in fracking, cementing and drilling of oil and gas wells; change file names as appropriate
datfrac = read_csv("Data/well_fracking_water_used.csv")
datcemt = read_csv("Data/well_cementing_water_used.csv")
datdril = read_csv("Data/well_drilling_water_used.csv")

# standardize unit of measurement of volume; change argument values as appropriate for original units of data
datfrac %<>% mutate(Volume = Volume * 42 / 1e6)  # 42 gallons per barrel; 1e6 gallons (gal) per 1 Mgal
datcemt %<>% mutate(Volume = Volume * 42 / 1e6)  # 42 gallons per barrel; 1e6 gallons (gal) per 1 Mgal
datdril %<>% mutate(Volume = Volume * 42 / 1e6)  # 42 gallons per barrel; 1e6 gallons (gal) per 1 Mgal


#  ------------------------------------------------------------------------#
# A.2. input to indirect water-use analysis----
#  ------------------------------------------------------------------------#


## NOTE:
#   1) to run the following code chunks (including sections B.2. and C.2.) as is, the data must be structured as follows:
#     a) rows of observations for individual water permits;
#     b) columns of variables for state, county, source (ground or surface), type (fresh or saline), use (water depot, etc.), relation to COG development (yes, no, maybe, NA, etc.), year and volume
#     c) see "permit_water_used.csv" for the appropriate data structure
#   2) the script may throw an error if data have the following limitations:
#     a) missing one or more of the listed variables;
#     b) lacking observations by individual well (for example, observations may be aggregated by county or by unit area)
#   3) the script can be modified as appropriate to run in the following conditions:
#     a) a COG play may have data available at a finer temporal resolution, such as by month
#     b) a COG play may have data available on some particular indirect uses at oil and gas wells, such as for dust control

# read data for water used by permits (and temporary water permits, etc.); change file names as appropriate
datperm = read_csv("Data/permit_water_used.csv")
dattemp = read_csv("Data/temporary_permit_water_used.csv")

# standardize unit of measurement of volume; change argument values as appropriate for original units of data
datperm %<>% mutate(Volume = Volume * 325851 / 1e6)  # 325851 gallons per acre-feet; 1e6 gallons (gal) per 1 Mgal
dattemp %<>% mutate(Volume = Volume * 325851 / 1e6)  # 325851 gallons per acre-feet; 1e6 gallons (gal) per 1 Mgal


#  ------------------------------------------------------------------------#
# A.3. input to ancillary water-use analysis----
#  ------------------------------------------------------------------------#


## NOTE:
#   1) the following data likely are the same as for the indirect water-use analysis;
#   2) see the preceding "NOTE" (section A.2.) for details about running the following code chunks

# read data for water used by permits (and temporary water permits, etc.); change file names as appropriate
datperm = read_csv("Data/permit_water_used.csv")
dattemp = read_csv("Data/temporary_permit_water_used.csv")

# standardize unit of measurement of volume; change argument values as appropriate for original units of data
datperm %<>% mutate(Volume = Volume * 325851 / 1e6)  # 325851 gallons per acre-feet; 1e6 gallons (gal) per 1 Mgal
dattemp %<>% mutate(Volume = Volume * 325851 / 1e6)  # 325851 gallons per acre-feet; 1e6 gallons (gal) per 1 Mgal

## NOTE:
#   1) to run the following code chunks (including sections B.3. and C.3.) as is, the data must be structured as follows:
#     a) rows of modeled climate values aggregated to county;
#     b) columns of variables for state, county, year, precipitation, and minimum and maximum temperature;
#     c) see "climate_modeled.csv" for the appropriate data structure
#   2) the data are unlikely to have limitations as they are uniformly available nationwide from the PRISM Climate Group
#   3) the script can be modified as appropriate if a COG play has data available at a finer spatial or temporal resolution

# read data for modeled climate
datclim = read_csv("Data/climate_modeled.csv")

## NOTE:
#   1) to run the following code chunks (including sections B.3. and C.3.) as is, the data must be structured as follows:
#     a) rows of estimated population values aggregated to county;
#     b) columns of variables for state, county, year and population;
#     c) see "population_estimated.csv" for the appropriate data structure
#   2) the data are unlikely to have limitations as they are uniformly available nationwide from the U.S. Census Bureau
#   3) the script can be modified as appropriate if a COG play has data available at a finer spatial resolution, such as by census tract or block

# read data for estimated population
datpopn = read_csv("Data/population_estimated.csv")



#  ------------------------------------------------------------------------#
# B.0. initialize process for water-use analysis----
#  ------------------------------------------------------------------------#


## NOTE:
#   1) if data are available preceding and following COG development, then run the following code chunk;
#   2) if not, then an appropriate year that COG development began must be selected using judgement

# model year that COG development began by estimating breakpoint in volume of water used
.brkpt = .model_breakpoint(data=datfrac)

## COMMENT:
#   a) if no breakpoint is estimated or multiple breakpoints are estimated, then judgment must be used to choose the appropriate year that COG development began
#   b) if a breakpoint is estimated, plot the water-use data (volume against year) to verify that the year is reasonable

## NOTE:
#   1) the water-use analyses (sections B.1., B.2., and B.3.) make confidence intervals around the parameter estimates
#   2) an appropriate confidence level must be chosen
.conlev=0.95


#  ------------------------------------------------------------------------#
# B.1. process for direct water-use analysis----
#  ------------------------------------------------------------------------#


## NOTE:
#   1) the following code chunk cannot be run with more than one data object per direct use
#   2) if a direct use has more than one data source available, then the following code chunk must be run again for each data source with the same use
#   3) the direct water-use analysis will involve analyses specific to each data source with the same use
#   4) output from the following function is also saved as a ".rds" file in the "Analyses" folder

# wrangle available direct water-use data to necessary structure for modeling; list available data in "data" argument
datdir = .wrangle_direct(data=list(datfrac, datcemt, datdril),
                         brkpt=.brkpt,
                         use=c("Fracking", "Cementing", "Drilling"),
                         type=c("Observed", "Estimated", "Estimated"))

## NOTE:
#   1) the following code chunk cannot be run with more than one data object per direct use
#   2) if a direct use has more than one data source available, then the following code chunk must be run again for each data source with the same use
#   3) the direct water-use analysis will involve analyses specific to each data source with the same use
#   4) output from the following function is also saved as a ".rds" file in the "Analyses" folder

# model mean and 10th, 50th, and 90th percentile water use for direct uses as a function of the number of wells using linear regression
datdir = read_rds("Analyses/datdir.rds")
moddir = .model_direct(data=datdir,
                       parm=c("Mean", "P10", "P50", "P90"),  # parm can also be "P01" through "P99"
                       use=c("Fracking", "Cementing", "Drilling", "Total_Direct"),
                       conlev=.conlev)


#  ------------------------------------------------------------------------#
# B.2. process for indirect water-use analysis----
#  ------------------------------------------------------------------------#


# wrangle available indirect water-use data to necessary structure for modeling; list available data in "data" argument
datind = .wrangle_indirect(data=list(datperm, dattemp),
                           data2=datdir,
                           brkpt=.brkpt,
                           cogrel="Yes",  # cogrel may also have "Maybe", etc.
                           wduse="Water Depot",  # wduse can also be NULL
                           conlev=.conlev)

## NOTE:
#   1) if the direct water-use analysis involves more than one data source per direct use, then the following code chunk must be run again for each data source
#   2) the indirect water-use analysis will involve analyses specific to each direct water-use analysis

# model mean and 10th, 50th, and 90th percentile water use for indirect uses as a function of the number of wells using linear regression
datind = read_rds("Analyses/datind.rds")
modind = .model_indirect(data=datind,
                         parm=c("Mean", "P10", "P50", "P90"),  # parm can also be "P01" through "P99"
                         use="Total_Indirect",
                         type=c("Fitted"),  # type may also have "Lower_Limit" and "Upper_Limit"
                         conlev=.conlev)


#  ------------------------------------------------------------------------#
# B.3. process for ancillary water-use analysis----
#  ------------------------------------------------------------------------#


# wrangle available ancillary water-use data to necessary structure for modeling; list available data in "data" argument
datanc = .wrangle_ancillary(data=list(datperm, dattemp),
                            data2=datdir,
                            data3=datclim,
                            brkpt=.brkpt,
                            cogrel="Yes",  # cogrel may also have "Maybe", etc.
                            wduse="Water Depot",  # wduse can also be NULL
                            conlev=.conlev)

## NOTE:
#   1) if the direct water-use analysis involves more than one data source per direct use, then the following code chunk must be run again for each data source
#   2) the ancillary water-use analysis will involve analyses specific to each direct water-use analysis

# model mean and 10th, 50th, and 90th percentile water use for ancillary uses as a function of the number of wells using linear regression
datanc = read_rds("Analyses/datanc.rds")
modanc = .model_ancillary(data=datanc,
                          parm=c("Mean", "P10", "P50", "P90"),  # parm can also be "P01" through "P99"
                          use=c("Domestic", "Industrial", "Irrigation", "Mining", "Public Supply", "Thermoelectric Power"),
                          type=c("Fitted"),  # type may also have "Lower_Limit" and "Upper_Limit"
                          conlev=.conlev)



#  ------------------------------------------------------------------------#
# B. process for population change analysis----
#  ------------------------------------------------------------------------#


## COMMENT:
#   the following function is not needed for the water-use analysis per se

# wrangle available population change data to necessary structure for modeling; list available data in "data" argument
datchng = .wrangle_change(data=list(datperm, dattemp),
                          data2=datdir,
                          data3=datpopn,
                          brkpt=.brkpt,
                          cogrel="Yes",  # cogrel may also have "Maybe", etc.
                          conlev=.conlev)

# model mean population change as a function of the number of wells using linear regression
datchng = read_rds("Analyses/datchng.rds")
modchng = .model_change(data=datchng,
                        conlev=.conlev)



#  ------------------------------------------------------------------------#
# C.1. output from direct water-use analysis----
#  ------------------------------------------------------------------------#


# write and plot data on direct water use associated with COG development
moddir = read_rds("Analyses/moddir.rds")
.visualize_direct(data=moddir,
                  comp="direct",
                  conlev=.conlev)

## NOTE:
#   1) data are structured as follows:
#     a) rows of estimated water-use coefficients by direct use;
#     b) columns of variables for parameter estimates and confidence intervals by parameter
#   2) data are plotted as direct water use against number of wells (grouped by year and county)


#  ------------------------------------------------------------------------#
# C.2. output from indirect water-use analysis----
#  ------------------------------------------------------------------------#


# write and plot data on indirect water use associated with COG development
modind = read_rds("Analyses/modind.rds")
.visualize_indirect(data=modind,
                    comp="indirect",
                    conlev=.conlev)

## NOTE:
#   1) data are structured as follows:
#     a) row of estimated water-use coefficients for indirect uses;
#     b) columns of variables for parameter estimates and confidence intervals by parameter
#   2) data are plotted as indirect water use against number of wells (grouped by year and county)


#  ------------------------------------------------------------------------#
# C.3. output from ancillary water-use analysis----
#  ------------------------------------------------------------------------#


# write and plot data on ancillary water use associated with COG development
modanc = read_rds("Analyses/modanc.rds")
.visualize_ancillary(data=modanc,
                     comp="ancillary",
                     conlev=.conlev)

## NOTE:
#   1) data are structured as follows:
#     a) row of estimated water-use coefficients by ancillary use;
#     b) columns of variables for parameter estimates and confidence intervals by parameter
#   2) data are plotted as ancillary water use against number of wells (grouped by year and county)


#  ------------------------------------------------------------------------#
# C. output for population change analysis----
#  ------------------------------------------------------------------------#


## COMMENT:
#   the following function is not needed for the water-use analysis per se

# write and plot data on population change associated with COG development
modchng = read_rds("Analyses/modchng.rds")
.visualize_change(data=modchng,
                  conlev=.conlev)

## NOTE:
#   1) data are structured as follows:
#     a) row of estimated population coefficient;
#     b) columns of variables for parameter estimate and confidence interval
#   2) data are plotted as population for against number of wells (grouped by year) and against year
